#!/usr/bin/env python3

import argparse
from xor_tools import plaintext_bytes
from xor_tools.rolling_key import RollingKey


def main():
    parser = argparse.ArgumentParser(description='Encrypts a file using a defined XOR key')
    parser.add_argument("-f", "--file", help="Input file", type=argparse.FileType('rb'), required=True)
    parser.add_argument("-o", "--output", help="Output file", type=argparse.FileType('wb'), required=True)
    parser.add_argument("-k", "--key", help="XOR key", type=plaintext_bytes, required=True)

    args = parser.parse_args()

    rk = RollingKey(args.key)
    for c, k in zip(args.file.read(), rk):
        args.output.write(bytes([c ^ k]))
    args.output.close()



#!/usr/bin/env python3

"""
Some examples:

xor_key_recovery -f samples/binary_elf_encrypted -p __libc_start_main -n 4
xor_key_recovery -f samples/binary_pe32_encrypted -p "This program cannot be run in DOS" -n 4
"""

import argparse
import sys
from xor_tools import plaintext_bytes
from xor_tools.rolling_key import RollingKey


def calc_template(text, keylength):
    return bytes([text[i] ^ text[i + keylength] for i in range(0, len(text) - keylength)])


def main():
    parser = argparse.ArgumentParser(description='Attempts to find XOR key using a known plaintext')
    parser.add_argument("-f", "--file", help="Input file (encrypted)", type=argparse.FileType('rb'), required=True)
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("-p", "--plaintext", help="Known plaintext", type=plaintext_bytes)
    group.add_argument("-pf", "--plaintext-file", help="Known plaintext", type=argparse.FileType('rb'))
    parser.add_argument("-n", "--keylength", help="Key length", type=int, required=True)
    parser.add_argument("-o", "--output", help="Output file", type=argparse.FileType('wb'))

    args = parser.parse_args()

    ciphertext = args.file.read()
    plaintext = args.plaintext if args.plaintext else args.plaintext_file.read()
    keylength = args.keylength

    if keylength < 1:
        print('[!] Key lenght must be larger than one.')
        sys.exit(1)

    if len(plaintext) < 2 * keylength:
        print('[!] Plaintext must be at least 2 times longer than expected keylength')
        sys.exit(2)

    plaintext_template = calc_template(plaintext, keylength)
    ciphertext_template = calc_template(ciphertext, keylength)

    try:
        index = ciphertext_template.index(plaintext_template)
    except ValueError:
        print('[-] Plaintext not found in ciphertext with provided keylength')
        sys.exit(3)

    print('[+] Found plaintext at position: {}'.format(index))

    rolling_key = RollingKey(bytes([ciphertext[index + i] ^ plaintext[i] for i in range(0, keylength)]))
    rolling_key = RollingKey(bytes([rolling_key[k] for k in range(-index, -index + keylength)]))
    print('[*] Key found: 0x{}'.format(rolling_key.key.hex()))

    if args.output:
        print('[+] Saving decrypted file to: {}'.format(args.output.name))
        args.output.write(bytes([c ^ k for c, k in zip(ciphertext, rolling_key)]))
        args.output.close()



#!/usr/bin/env python3

"""
Module containing RollingKey class, implementing
key, that could be used for encryption element by element.

Example:
    rk = RollingKey(b'ABCD')
    ciphertext = [ c^k for c,k in zip(plaintext, rk) ]
"""


class RollingKey:
    """
     Class implementing rolling key, that could be used
    over and over for encrypting with repeated key.

    WARNING: Rolling codes should NOT be used for implementations
             where high security is important component (like OTP).
    """

    def __init__(self, key):
        self.key = key
        self.index = 0

    def __iter__(self):
        return self

    def __next__(self):
        if self.index >= len(self.key):
            self.index = 0
        tmp = self.key[self.index]
        self.index += 1
        return tmp

    def __getitem__(self, index):
        return self.key[index % len(self.key)]

    def __str__(self):
        if len(self.key) > 10:
            return "RollingKey(%s ...)" % self.key[:10]
        else:
            return "RollingKey(%s)" % self.key[:10]


def test():
    rk = RollingKey(b'ABCD')

    assert next(rk) == 65
    assert next(rk) == 66
    assert next(rk) == 67
    assert next(rk) == 68
    assert next(rk) == 65

    assert rk[1000] == 65


if __name__ == "__main__":
    test()



import os
import setuptools


# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setuptools.setup(
    name="xor_tools",
    version="1.0.1",
    python_requires='>3.5.2',
    url='https://github.com/lukaskuzmiak/xor_tools',
    author="Lukas Kuzmiak, nshadov",
    author_email="lukash@backstep.net",
    description="XOR Tools",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    license="MIT",
    keywords="xor key recovery encrypt decrypt",
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': [
            'xor_key_recovery=xor_tools.xor_key_recovery:main',
            'xor_encrypt_file=xor_tools.xor_encrypt_file:main'
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)
